package PowerEditContentData::CMS;

use strict;
use warnings;
use PowerEditContentData::ContentData qw(make_list_actions);
use MT::Util qw( format_ts relative_date remove_html encode_html encode_js
    encode_url archive_file_for offset_time_list break_up_text first_n_words );
use MT::I18N qw( const wrap_text );

use base 'Exporter';
our @EXPORT_OK = qw( rest_day is_limit );

sub open_batch_editor {
    my $app = shift;
    my ($param) = @_;
    $param ||= {};

    my $plugin = MT->component("PowerEditContentData");
    return $app->error( $plugin->translate("expiration of a term.") )
        if is_limit();

    my @ids = $app->multi_param('id')
        or return "Invalid request.";
    my %dupe;
    @ids = grep { !$dupe{$_}++ } @ids;

    require MT::ContentData;
    my $type = $app->param('_type');
    my $pkg = $app->model($type) or return "Invalid request.";

    my $blog_id = $app->param('blog_id');
    return $app->return_to_dashboard( redirect => 1 )
        unless $blog_id;
    my $blog = $app->model('blog')->load($blog_id);
    return $app->return_to_dashboard( redirect => 1 )
        unless $blog;

    my @blog_ids;
#    my $action
#        = $type eq 'page'
#        ? 'open_batch_page_editor_via_list'
#        : 'open_batch_entry_editor_via_list';
    if ( $blog->is_blog ) {
#        return $app->permission_denied()
#            unless $app->can_do($action);
        push @blog_ids, $blog_id;
    }
    else {
        my $blogs = $blog->blogs;
        @blog_ids = map { $_->id } @$blogs;
        push @blog_ids, $blog->id;
    }

    if ( !$app->user->is_superuser ) {
        my $terms = {
            author_id => $app->user->id,
            blog_id   => \' = cd_blog_id',
        };
#        my $perms = MT::Permission->load_permissions_from_action($action);
#        if ($perms) {
#            my @perms_term;
#            foreach my $perm (@$perms) {
#                $perm =~ m/\.(.*)/;
#                push @perms_term, '-or' if @perms_term;
#                push @perms_term, { permissions => { like => "%'$1'%" } };
#            }
#            $terms = [ $terms, '-and', \@perms_term, ];
#        }

        my $cnt = $pkg->count(
            {   id      => \@ids,
                blog_id => \@blog_ids
            },
            {   join => MT::Permission->join_on(
                    undef, $terms, { unique => 1, }
                ),
                unique => 1,
            }
        );

#        return $app->permission_denied()
#            if scalar @ids != $cnt;
    }

    # Loading objects
    my $iter = $pkg->load_iter(
        { class => $type, id => \@ids, blog_id => \@blog_ids },
        { sort => 'authored_on', direction => 'descend' }
    );

    my $list_pref = $app->list_pref($type);
    my %param     = %$list_pref;
    $param{has_expanded_mode} = 0;
    delete $param{view_expanded};

    my $data = build_content_data_table(
        $app,
        iter          => $iter,
        is_power_edit => 1,
        param         => \%param,
        type          => $type
    );
    delete $_->{object} foreach @$data;
    delete $param{entry_table} unless @$data;

    $param{saved}               = $app->param('saved');
    $param{object_type}         = $type;
#    $param{object_label}        = $pkg->class_label;
    $param{object_label_plural} = $param{search_label}
        = $pkg->class_label_plural;
    $param{list_start}            = 1;
    $param{list_end}              = scalar @$data;
    $param{listing_screen}        = 1;
    $param{blog_view}             = 1;
    $param{container_label}       = $pkg->container_label;
    $param{mode}                  = "save_content_datas";
    $param{sitepath_unconfigured} = $blog->site_path ? 0 : 1;

    $param->{return_args} ||= $app->make_return_args;
    my @return_args
        = grep { $_ !~ /^(?:offset|__mode|id)=/ } split /&/,
        $param->{return_args};
    push @return_args, '__mode=open_batch_editor';
    push @return_args, "id=$_" foreach (@ids);

    $param{return_args} = join '&', @return_args;
    $param{screen_id}   = "batch-edit-entry";
    $param{screen_id}   = "batch-edit-page"
        if $param{object_type} eq "page";
    $param{screen_class} .= " batch-edit";

    $param{ rest_day } = $plugin->translate('can use [_1]', rest_day());

    $app->load_tmpl( "edit_content_data_batch.tmpl", \%param );
}

sub build_content_data_table {
    my $app = shift;
    my (%args) = @_;

    my $app_author = $app->user;
    my $perms      = $app->permissions
        or return $app->permission_denied();
    my $type  = $args{type};
    my $class = $app->model($type);

    my $list_pref = $app->list_pref($type);
    if ( $args{is_power_edit} ) {
        delete $list_pref->{view_expanded};
    }
    my $iter;
    if ( $args{load_args} ) {
        $iter = $class->load_iter( @{ $args{load_args} } );
    }
    elsif ( $args{iter} ) {
        $iter = $args{iter};
    }
    elsif ( $args{items} ) {
        $iter = sub { shift @{ $args{items} } };
    }
    return [] unless $iter;
    my $limit         = $args{limit};
    my $is_power_edit = $args{is_power_edit} || 0;
    my $param         = $args{param} || {};

    ## Load list of categories for display in filter pulldown (and selection
    ## pulldown on power edit page).
    my ( $c_data, %cats );
    my $blog_id = $app->param('blog_id');
#    if ($blog_id) {
#        $c_data = $app->_build_category_list(
#            blog_id => $blog_id,
#            type    => $class->container_type,
#        );
#        my $i = 0;
#        for my $row (@$c_data) {
#            $row->{category_index} = $i++;
#            my $spacer = $row->{category_label_spacer} || '';
#            $spacer =~ s/\&nbsp;/\\u00A0/g;
#            $row->{category_label_js}
#                = $spacer . encode_js( $row->{category_label} );
#            $cats{ $row->{category_id} } = $row;
#        }
#        $param->{category_loop} = $c_data;
#    }

    my ( $date_format, $datetime_format );

    if ($is_power_edit) {
        $date_format     = "%Y.%m.%d";
        $datetime_format = "%Y-%m-%d %H:%M:%S";
    }
    else {
        $date_format     = MT::App::CMS::LISTING_DATE_FORMAT();
        $datetime_format = MT::App::CMS::LISTING_DATETIME_FORMAT();
    }

    my @cat_list;
#    if ($is_power_edit) {
#        @cat_list = sort {
#            $cats{$a}->{category_index} <=> $cats{$b}->{category_index}
#            }
#            keys %cats;
#    }

    my @data;
    my %blogs;
    require MT::Blog;
    my $title_max_len = const('DISPLAY_LENGTH_EDIT_ENTRY_TITLE');
    my $excerpt_max_len
        = const('DISPLAY_LENGTH_EDIT_ENTRY_TEXT_FROM_EXCERPT');
    my $text_max_len = const('DISPLAY_LENGTH_EDIT_ENTRY_TEXT_BREAK_UP');
    my %blog_perms;
    $blog_perms{ $perms->blog_id } = $perms if $perms;

    while ( my $obj = $iter->() ) {
        my $blog_perms;
        if ( !$app_author->is_superuser() ) {
            $blog_perms = $blog_perms{ $obj->blog_id }
                || $app_author->blog_perm( $obj->blog_id );
        }

        my $row = $obj->get_values;
        $row->{text} ||= '';
        if ( my $ts = $obj->authored_on ) {
            $row->{created_on_formatted}
                = format_ts( $date_format, $ts, $obj->blog,
                $app->user ? $app->user->preferred_language : undef );
            $row->{created_on_time_formatted}
                = format_ts( $datetime_format, $ts, $obj->blog,
                $app->user ? $app->user->preferred_language : undef );
            $row->{created_on_relative}
                = relative_date( $ts, time, $obj->blog );
        }
        if ( my $ts = $obj->modified_on ) {
            $row->{modified_on_formatted}
                = format_ts( $date_format, $ts, $obj->blog,
                $app->user ? $app->user->preferred_language : undef );
            $row->{modified_on_time_formatted}
                = format_ts( $datetime_format, $ts, $obj->blog,
                $app->user ? $app->user->preferred_language : undef );
            $row->{modified_on_relative}
                = relative_date( $ts, time, $obj->blog );
        }
        my $author = $obj->author;
        $row->{author_name}
            = $author ? $author->name : $app->translate('(user deleted)');
#        if ( my $cat = $obj->category ) {
#            $row->{category_label}    = $cat->label;
#            $row->{category_basename} = $cat->basename;
#            $row->{category_id}       = $cat->id;
#        }
#        else {
#            $row->{category_label}    = '';
#            $row->{category_basename} = '';
#            $row->{category_id}       = '';
#        }
        $row->{file_extension} = $obj->blog ? $obj->blog->file_extension : '';
        $row->{title_short} = $obj->label;
#        if ( !defined( $row->{title_short} ) || $row->{title_short} eq '' ) {
#            my $title = remove_html( $obj->text );
#            $row->{title_short}
#                = substr( defined($title) ? $title : "", 0, $title_max_len )
#                . '...';
#        }
#        else {
            $row->{title_short} = remove_html( $row->{title_short} );
            $row->{title_short}
                = substr( $row->{title_short}, 0, $title_max_len + 3 ) . '...'
                if length( $row->{title_short} ) > $title_max_len;
#        }
#        if ( $row->{excerpt} ) {
#            $row->{excerpt} = remove_html( $row->{excerpt} );
#        }
        if ( !$row->{excerpt} ) {
            my $text = remove_html( $row->{text} ) || '';
            $row->{excerpt} = first_n_words( $text, $excerpt_max_len );
            if ( length($text) > length( $row->{excerpt} ) ) {
                $row->{excerpt} .= ' ...';
            }
        }
        $row->{text} = break_up_text( $row->{text}, $text_max_len )
            if $row->{text};
        $row->{title_long} = remove_html( $obj->label );
        $row->{status_text}
            = $app->translate( MT::ContentStatus::status_text( $obj->status ) );
        $row->{ "status_" . MT::ContentStatus::status_text( $obj->status ) } = 1;
        $row->{has_edit_access} = $app_author->is_superuser
            || ( ( 'entry' eq $type )
            && $blog_perms
            && $blog_perms->can_edit_entry( $obj, $app_author ) )
            || (
               ( 'page' eq $type )
            && $blog_perms
            && (  $app_author->id == $obj->author_id
                ? $blog_perms->can_do('edit_own_page')
                : $blog_perms->can_do('edit_all_pages')
            )
            );
        if ($is_power_edit) {
            $row->{has_publish_access} = $app_author->is_superuser
                || ( $blog_perms
                && $blog_perms->can_edit_entry( $obj, $app_author, 1 ) );
            $row->{is_editable} = $row->{has_edit_access};

            ## This is annoying. In order to generate and pre-select the
            ## category, user, and status pull down menus, we need to
            ## have a separate *copy* of the list of categories and
            ## users for every entry listed, so that each row in the list
            ## can "know" whether it is selected for this entry or not.
            my @this_c_data;
#            my $this_category_id
#                = $obj->category ? $obj->category->id : undef;
#            for my $c_id (@cat_list) {
#                push @this_c_data, { %{ $cats{$c_id} } };
#                $this_c_data[-1]{category_is_selected} = $this_category_id
#                    && $this_category_id == $c_id ? 1 : 0;
#            }
#            $row->{row_category_loop} = \@this_c_data;

            if ( $obj->author ) {
                $row->{row_author_name} = $obj->author->name;
                $row->{row_author_id}   = $obj->author->id;
            }
            else {
                $row->{row_author_name}
                    = $app->translate( '(user deleted - ID:[_1])',
                    $obj->author_id );
                $row->{row_author_id} = $obj->author_id,;
            }

            $row->{entry_blog_id} = $obj->blog_id;
        }
        if ( my $blog = $blogs{ $obj->blog_id }
            ||= MT::Blog->load( $obj->blog_id ) )
        {
            $row->{weblog_id}   = $blog->id;
            $row->{weblog_name} = $blog->name;
        }
        if ( $obj->status == MT::ContentStatus::RELEASE() ) {
            $row->{entry_permalink}
                = MT::Util::encode_html( $obj->permalink );
        }
        $row->{object} = $obj;
        push @data, $row;
    }
    return [] unless @data;

    $param->{entry_table}[0] = {%$list_pref};
    $param->{object_loop} = $param->{entry_table}[0]{object_loop} = \@data;
    $app->load_list_actions( $type, \%$param )
        unless $is_power_edit;
    \@data;
}

sub save_content_datas {
    my $app = shift;

    my $plugin = MT->component("PowerEditContentData");
    return $app->error( $plugin->translate("expiration of a term.") )
        if is_limit();

    require MT::Util::Log;
    MT::Util::Log::init();

    MT::Util::Log->info('--- Start save_entries.');

    my $perms = $app->permissions
        or $app->permission_denied();
    my $type = $app->param('_type') || '';
    my $blog_id = $app->param('blog_id');
    return $app->return_to_dashboard( redirect => 1 )
        unless $blog_id;
    my $blog = $app->model('blog')->load($blog_id);
    return $app->return_to_dashboard( redirect => 1 )
        unless $blog;

#    MT::Util::Log->info(' Start permission check.');

PERMCHECK: {
        my $action = 'save_multiple_content_data';
#            = $type eq 'page'
#            ? 'save_multiple_pages'
#            : 'save_multiple_entries';
        if ($blog_id) {
            if ( $blog->is_blog ) {
                last PERMCHECK if $app->can_do($action);
            }
            else {
                my $blogs    = $blog->blogs;
                my $blog_ids = [ $blog->id ];
                my @map      = map { $_->id } @$blogs;
                push @$blog_ids, map { $_->id } @{ $blog->blogs };

                last PERMCHECK
                    if $app->user->can_do(
                    $action,
                    blog_id      => $blog_ids,
                    at_least_one => 1,
                    );
            }
        }
        else {
            last PERMCHECK
                if $app->user->can_do( $action, at_least_one => 1 );
        }
        return $app->permission_denied();
    }

#    MT::Util::Log->info(' End   permission check.');

    $app->validate_magic() or return;

    my @p = $app->multi_param;
    require MT::ContentData;
    require MT::Log;
    my $this_author    = $app->user;
    my $this_author_id = $this_author->id;
    my @objects;

    MT::Util::Log->info(' Start check params.');

    for my $p (@p) {
        next unless $p =~ /^author_id_(\d+)/;
        my $id    = $1;
        my $entry = MT::ContentData->load($id)
            or next;
        my $old_status = $entry->status;
        my $orig_obj   = $entry->clone;
        $perms = $app->user->permissions( $entry->blog_id );
#        if ( $perms->can_edit_entry( $entry, $this_author ) ) {
            my $author_id = $app->param( 'author_id_' . $id );
            my $label     = $app->param( 'label_' . $id )
                || $app->param( 'no_label_' . $id );
            $entry->author_id( $author_id ? $author_id : 0 );
            $entry->label($label);
#        }
#        else {
#            return $app->permission_denied();
#        }
        if ( $perms->can_edit_entry( $entry, $this_author, 1 ) )
        {    ## can he/she change status?
            my $status = $app->param( 'status_' . $id );
            $entry->status($status);

            my $date_closure = sub {
                my ( $val, $col, $name ) = @_;

                # FIXME: Should be assigning the publish_date field here
                my $ts = MT::Util::valid_date_time2ts($val)
                    or return $app->error(
                    $app->translate(
                        "Invalid date '[_1]'; [_2] dates must be in the format YYYY-MM-DD HH:MM:SS.",
                        $val,
                        $name
                    )
                    );

                if ( $col eq 'authored_on' && $entry->unpublished_on ) {
                    require MT::DateTime;
                    return $app->error(
                        $app->translate(
                            "Invalid date '[_1]'; 'Published on' dates should be earlier than the corresponding 'Unpublished on' date '[_2]'.",
                            $val,
                            format_ts(
                                "%Y-%m-%d %H:%M:%S",
                                $entry->unpublished_on,
                                $blog,
                                $app->user
                                ? $app->user->preferred_language
                                : undef
                            )
                        )
                        )
                        if (
                        MT::DateTime->compare(
                            blog => $blog,
                            a    => $ts,
                            b    => $entry->unpublished_on
                        ) > 0
                        );
                }

                $entry->$col($ts);
            };

            my $co = $app->param( 'created_on_' . $id ) || '';
            $date_closure->(
                $co, 'authored_on', MT->translate('authored on')
            ) or return;
            $co = $app->param( 'modified_on_' . $id ) || '';
            $date_closure->(
                $co, 'modified_on', MT->translate('modified on')
            ) or return;

            # Clear Unpublish Date
            if (   $old_status == MT::ContentStatus::UNPUBLISH()
                && $entry->status == MT::ContentStatus::RELEASE() )
            {
                $entry->unpublished_on(undef);
            }
        }
#        $app->run_callbacks( 'cms_pre_save.' . $type,
#            $app, $entry, $orig_obj )
#            || return $app->error(
#            $app->translate(
#                "Saving [_1] failed: [_2]", $entry->label,
#                $app->errstr
#            )
#            );
        $entry->save
            or return $app->error(
            $app->translate(
                "Saving entry '[_1]' failed: [_2]", $entry->label,
                $entry->errstr
            )
            );

        # If Status was changed from PUBLISH to others,
        # Let's remove archive files.
        if (   ( $entry->status || 0 ) != MT::ContentStatus::RELEASE()
            && $old_status == MT::ContentStatus::RELEASE()
            && $app->config('DeleteFilesAtRebuild') )
        {
            if ( my $blog = $entry->blog ) {
#                my $archive_type = 'entry' eq $type ? 'Individual' : 'Page';
#                my $file = archive_file_for( $entry, $blog, $archive_type );
                $app->publisher->remove_content_data_archive_file(
                    ContentData       => $entry,
#                    ArchiveType => $archive_type,
                );
            }
        }

#        my $cat_id = $app->param("category_id_$id");
#        my $place  = MT::Placement->load(
#            {   entry_id   => $id,
#                is_primary => 1
#            }
#        );
#        if ( $place && !$cat_id ) {
#            $place->remove
#                or return $app->error(
#                $app->translate(
#                    "Removing placement failed: [_1]",
#                    $place->errstr
#                )
#                );
#        }
#        elsif ($cat_id) {
#            unless ($place) {
#                $place = MT::Placement->new;
#                $place->entry_id($id);
#                $place->blog_id($blog_id);
#                $place->is_primary(1);
#            }
#            $place->category_id($cat_id);
#            $place->save
#                or return $app->error(
#                $app->translate(
#                    "Saving placement failed: [_1]",
#                    $place->errstr
#                )
#                );
#        }
        my $message;
        if ( $orig_obj->status ne $entry->status ) {
            $message = $app->translate(
                "[_1] '[_2]' (ID:[_3]) edited and its status changed from [_4] to [_5] by user '[_6]'",
                $entry->class_label,
                $entry->label,
                $entry->id,
                $app->translate(
                    MT::ContentStatus::status_text( $orig_obj->status )
                ),
                $app->translate( MT::ContentStatus::status_text( $entry->status ) ),
                $this_author->name
            );
        }
        else {
            $message
                = $app->translate(
                "[_1] '[_2]' (ID:[_3]) edited by user '[_4]'",
                'content_data', $entry->label, $entry->id,
                $this_author->name );
        }
        $app->log(
            {   message  => $message,
                level    => MT::Log::INFO(),
                class    => 'content_data',
                category => 'edit',
                metadata => $entry->id
            }
        );
        push( @objects, { current => $entry, original => $orig_obj } );
    }

    MT::Util::Log->info(' End   check params.');

    MT::Util::Log->info(' Start callbacks cms_post_bulk_save.');

    $app->run_callbacks(
        'cms_post_bulk_save.' . ( $type eq 'entry' ? 'entries' : 'pages' ),
        $app, \@objects );

    MT::Util::Log->info(' End   callbacks cms_post_bulk_save.');

    $app->add_return_arg( 'saved' => 1, is_power_edit => 1 );

    MT::Util::Log->info('--- End   save_entries.');

    $app->call_return;
}

sub init_request {
    my $plugin = MT->component("PowerEditContentData");
    my $csv_column = $plugin->get_config_value('pec_column_cd', 'system') || ',';
    if ($csv_column eq ',') {
        $plugin->set_config_value('pec_column_cd', time(), 'system');
    }
    make_list_actions();
}

sub is_limit {
    return (rest_day() > 0) ? 0 : 1;
}

sub rest_day {
    my $plugin = MT->component("PowerEditContentData");
    my $csv_column = $plugin->get_config_value('pec_column_cd', 'system') || '';
    my $duration = time() - $csv_column;
    my $rest_day = 31 - int($duration / (24 * 3600));
    return ($rest_day > 0) ? $rest_day : 0;
}


1;
